from re import I

from django.conf import settings
from django.urls import path
from rest_framework.schemas import get_schema_view
from rest_framework.settings import api_settings

from . import settings
from .schemas import OpenApiSchemaGenerator

app_name = 'openapi'

urlpatterns = []

if getattr(settings, 'OPENAPI_ENABLE', False):
    urlpatterns.append(
        path('openapi', get_schema_view(
            title=settings.OPENAPI_TITLE,
            description=settings.OPENAPI_DESCRIPTION,
            version=settings.OPENAPI_VERSION,
            public=settings.OPENAPI_PUBLIC,
            renderer_classes=api_settings.DEFAULT_RENDERER_CLASSES,
            generator_class=OpenApiSchemaGenerator
        ), name='openapi-schema')
    )



from rest_framework.permissions import (SAFE_METHODS, IsAuthenticated,
                                        IsAuthenticatedOrReadOnly)
from rest_framework.schemas.openapi import AutoSchema, SchemaGenerator

from . import constants, settings


class OpenApiSchemaGenerator(SchemaGenerator):

    def get_server(self, request):
        servers = settings.OPENAPI_SERVERS
        if len(servers) == 0:
            servers = [
                {"url": f"{request.scheme}://{request.get_host()}"}
            ]
        return servers

    def get_schema(self, *args, **kwargs):
        schema = super().get_schema(*args, **kwargs)
        schema["servers"] = self.get_server(args[0])
        schema["tags"] = settings.OPENAPI_TAGS
        schema["components"]["securitySchemes"] = settings.OPENAPI_SECURITY_SCHEMES
        schema["components"]["schemas"]["ErrorOnSend"] = constants.ERROR_ON_SEND_LAYOUT
        return schema


class OpenApiSchema(AutoSchema):

    def get_operation_id(self, path, method):
        operation_id = getattr(self.view, 'openapi_id', None)
        if hasattr(self.view, 'openapi_mount_id'):
            return self.view.openapi_mount_id(path, method)
        return operation_id if operation_id else super().get_operation_id(path, method)

    def map_serializer(self, serializer):
        results = super().map_serializer(serializer)
        
        append_required = getattr(self.view, 'openapi_append_required_fields', None)
        if append_required:
            assert isinstance(append_required, (list, tuple)), "`openapi_append_required_fields` must be list or tuple."
            results['required'] = [*apend_required, *results['required']]

        return results

    def get_description(self, path, method):
        description = getattr(self.view, 'openapi_description', None)
        if hasattr(self.view, 'openapi_mount_description'):
            return self.view.openapi_mount_description(path, method)
        return description if description else super().get_description(path, method)

    def get_responses(self, path, method):
        responses = super().get_responses(path, method)

        status_code = getattr(self.view, 'success_status_code', None)
        if status_code:
            responses = {f'{status_code}':  responses[list(responses.keys())[0]]}

        view_permissions = getattr(self.view, 'permission_classes', [None, ])
        has_auth_class = len(view_permissions) > 0
        safe_path = path not in SAFE_METHODS
        if has_auth_class and issubclass(view_permissions[0], IsAuthenticated):
            responses['401'] = {"description": "Authentication credentials were not provided."}

        if has_auth_class and issubclass(view_permissions[0], IsAuthenticatedOrReadOnly) and safe_path:
            responses['401'] = {"description": "Authentication credentials were not provided."}

        if method in ['POST', 'PUT', 'PATCH']:
            responses['400'] = {
                "description": "One or more filled fields or bussiness logic are invalid.",
                "content": {
                    ct: {'schema': {'$ref': '#/components/schemas/ErrorOnSend'}}
                    for ct in self.request_media_types
                }
            }

        if getattr(self.view, 'openapi_enable_404_status_code', False):
            responses['404'] = {"description": "Object or entity not found."}

        # view_permissions = getattr(self.view, 'permission_classes', [None, ])
        # if issubclass(view_permissions[0], IsAuthenticated):
        #     responses['403'] = {"description": "You are not allowed to perform this action."}

        return responses

    def get_components(self, path, method):
        components = super().get_components(path, method)
        return components

    def get_operation(self, path, method):
        operation = super().get_operation(path, method)
        
        view_permissions = getattr(self.view, 'permission_classes', [None, ])
        has_auth_class = len(view_permissions) > 0
        if has_auth_class and issubclass(view_permissions[0], IsAuthenticated):
            security = settings.OPENAPI_OPERATOR_SECURITY
            if security is None:
                security = [{x: []} for x in list(settings.OPENAPI_SECURITY_SCHEMES.keys())]
            operation['security'] = security

        return operation



ERROR_ON_SEND_LAYOUT = {
    "type": "object",
    "properties": {
        "non_field_errors": {
            "type": "array",
            "items": {
                "type": "string",
            },
            "description": "Contains errors on bussiness logic (Ex. repeated data)"
        },
        "field_name": {
            "type": "array",
            "items": {
                "type": "string",
            },
            "description": "Contains errors on field (Ex. invalid type) - OBS: each field with error generate a line"
        },
    }
}



from django.views.generic import TemplateView

from .settings import OPENAPI_URL, TEMPLATE_TITLE


class SwaggerView(TemplateView):
    template_name = 'swagger/swagger.html'

    def get_context_data(self, **kwargs):
        return {
            'OPENAPI_URL': OPENAPI_URL,
            'TITLE': TEMPLATE_TITLE
        }



from pydoc import locate

from django.conf import settings
from django.urls import reverse_lazy

_CONFIG = getattr(settings, 'SWAGGER_CONFIGURATION', {})


SWAGGER_ENABLE = getattr(settings, 'SWAGGER_ENABLE', False)
TEMPLATE_TITLE = _CONFIG.get('TEMPLATE_TITLE', 'ReDoc')
OPENAPI_URL = _CONFIG.get('OPENAPI_URL') or reverse_lazy('openapi:openapi-schema')
SWAGGER_VIEW = locate(_CONFIG.get('SWAGGER_VIEW', 'djangorest_openapi_utils.swagger.view.SwaggerView.as_view'))



from pydoc import locate

from django.conf import settings

from . import constants

_CONFIG = getattr(settings, 'OPENAPI_CONFIGURATION', {})

OPENAPI_ENABLE = getattr(settings, 'OPENAPI_ENABLE', False)

# Instance parameters
OPENAPI_TITLE = _CONFIG.get('TITLE', 'OpenApi URL')
OPENAPI_DESCRIPTION = _CONFIG.get('DESCRIPTION', 'Fast Django REST OpenApi')
OPENAPI_VERSION = _CONFIG.get('VERSION', '1.0.0 - Beta')
OPENAPI_PUBLIC = _CONFIG.get('PUBLIC', False)

OPENAPI_TAGS = _CONFIG.get('TAGS', [])
OPENAPI_SERVERS = _CONFIG.get('SERVERS', [])
# see https://swagger.io/docs/specification/authentication/
OPENAPI_SECURITY_SCHEMES = _CONFIG.get('SECURITY_SCHEMES', {})
OPENAPI_OPERATOR_SECURITY = _CONFIG.get('OPERATOR_SECURITY', None)
OPENAPI_ERROR_ON_SEND_LAYOUT = _CONFIG.get('ERROR_ON_SEND_LAYOUT', constants.ERROR_ON_SEND_LAYOUT)


def get_openapi_tags():
    return OPENAPI_TAGS


def set_openapi_tags(name, description):
    OPENAPI_TAGS.append({"name": name, "description": description})



from django.urls import path

from .settings import RAPIDOC_ENABLE, RAPIDOC_VIEW

app_name = 'rapidoc'

urlpatterns = []

if RAPIDOC_ENABLE:
    urlpatterns.append(
        path('rapidoc', RAPIDOC_VIEW(), name='rapidoc-template')
    )



from django.urls import path

from .settings import REDOC_ENABLE, REDOC_VIEW

app_name = 'redoc'

urlpatterns = []

if REDOC_ENABLE:
    urlpatterns.append(
        path('redoc', REDOC_VIEW(), name='redoc-template')
    )



from pydoc import locate

from django.conf import settings
from django.urls import reverse_lazy

_CONFIG = getattr(settings, 'REDOC_CONFIGURATION', {})


REDOC_ENABLE = getattr(settings, 'REDOC_ENABLE', False)
TEMPLATE_TITLE = _CONFIG.get('TEMPLATE_TITLE', 'ReDoc')
OPENAPI_URL = _CONFIG.get('OPENAPI_URL') or reverse_lazy('openapi:openapi-schema')
REDOC_VIEW = locate(_CONFIG.get('REDOC_VIEW', 'djangorest_openapi_utils.redoc.view.ReDocView.as_view'))



from django.urls import path

from .settings import SWAGGER_ENABLE, SWAGGER_VIEW

app_name = 'swagger'

urlpatterns = []

if SWAGGER_ENABLE:
    urlpatterns.append(
        path('swagger', SWAGGER_VIEW(), name='swagger-template')
    )
